#include "runtime.h"
#include "config.h"
#include "aio.h"
#include "file_tab.h"
#include "buffer_manager.h"
#include "uidtab.h"

extern rt_cvar rt_Renderer;
extern rt_cvar rt_Fullscreen;
extern rt_cvar rt_WindowWidth;
extern rt_cvar rt_WindowHeight;
extern rt_cvar rt_BufferManagerMemory;
extern rt_cvar rt_FileTabCapacity;
extern rt_cvar rt_MaxConcurrentAsyncIO;
extern rt_cvar rt_AssetCacheSize;

void RegisterRuntimeCVars(void) {
    rtRegisterCVAR(&rt_Renderer);
    rtRegisterCVAR(&rt_Fullscreen);
    rtRegisterCVAR(&rt_WindowWidth);
    rtRegisterCVAR(&rt_WindowHeight);
    rtRegisterCVAR(&rt_BufferManagerMemory);
    rtRegisterCVAR(&rt_FileTabCapacity);
    rtRegisterCVAR(&rt_MaxConcurrentAsyncIO);
    rtRegisterCVAR(&rt_AssetCacheSize);
}

extern void SetMainThreadId(void);

extern rt_result InitBufferManager(void);
extern void ShutdownBufferManager(void);
extern rt_result InitFileTab(void);
extern void ShutdownFileTab(void);
extern rt_result InitAIO(void);
extern void ShutdownAIO(void);
extern rt_result InitAssetCache(void);
extern void ShutdownAssetCache(void);

extern rt_result LoadUIDTable(void);
extern void ReleaseUIDTable(void);
extern rt_result LoadAssetDependencies(void); 
extern void ReleaseAssetDependencies(void);
extern rt_result LoadPackageNames(void);

RT_DLLEXPORT rt_result rtInitRuntime(void) {
    SetMainThreadId();
    RegisterRuntimeCVars();

    rt_result res;
    if ((res = InitBufferManager()) != RT_SUCCESS) {
        rtReportError("BUFFERMGR", "Init failed.");
        return res;
    }

    if ((res = InitFileTab()) != RT_SUCCESS) {
        rtReportError("FTAB", "Init failed.");
        return res;
    }

    if ((res = InitAIO()) != RT_SUCCESS) {
        rtReportError("AIO", "Init failed.");
        return res;
    }

    if ((res = InitAssetCache()) != RT_SUCCESS) {
        rtReportError("ASSETCACHE", "Init failed.");
        return res;
    }

    if ((res = LoadUIDTable()) != RT_SUCCESS) {
        rtLog("CORE", "LoadUIDTable returned result: %u", res);
    }

    if ((res = LoadAssetDependencies()) != RT_SUCCESS) {
        rtReportError("ASSETDEP", "Init failed.");
        return res;
    }

    if ((res = LoadPackageNames()) != RT_SUCCESS) {
        rtLog("CORE", "LoadPackageNames returned result: %u", res);
    }

    return RT_SUCCESS;
}

RT_DLLEXPORT void rtShutdownRuntime(void) {
    ReleaseAssetDependencies();
    ReleaseUIDTable();
    ShutdownAssetCache();
    ShutdownAIO();
    ShutdownFileTab();
    ShutdownBufferManager();
}